import json

import falcon

from restapi.apps.evo.utils import execute_command
from restapi.utils.misc import sanitize
from restapi.utils.logs import get_logger

_LOG = get_logger()
_IFDOWN = '/sbin/ifdown'
_IFRELOAD = '/sbin/ifreload'
_IFUP = '/sbin/ifup'
_IP = '/sbin/ip'


class LacpCommand(object):

    def on_get(self, req, resp, groupName):
        """
        Command:
        Returns the contents of /proc/net/bonding/{groupName}

        URI (GET):
        intf/lacp/{dev}

        Response Body:
        Contents of /proc/net/bonding/{groupName}

        Response status code:
        200: Success
        """
        resp.status = falcon.HTTP_200
        file_path = '/proc/net/bonding/%s' % groupName
        _LOG.debug('Reading contents of %s', file_path)
        try:
            with open(file_path, 'r') as fd:
                resp.body = json.dumps({
                    'output': sanitize(fd.read(-1))
                })
        except Exception as ex:
            resp.body = json.dumps({
                'error': str(ex)
            })


class IfCommand(object):

    ACTION_DOWN = 'down'
    ACTION_RELOAD = 'reload'
    ACTION_UP = 'up'

    DEPENDS_INTERFACES = 'interfaces'
    DEPENDS_VLAN = 'vlan'

    __ACTION_VALUES = (ACTION_DOWN, ACTION_RELOAD, ACTION_UP)
    __DEPENDS_VALUES = (DEPENDS_INTERFACES, DEPENDS_VLAN)

    def on_post(self, req, resp, devName=None):
        """
        Command:
        ifreload -a
        ifup --with-depends {interfaces}
        ifdown {lacpGroup}
        ifdown --with-depends {vlan}

        URI:
        intf/state
        intf/state/{dev}

        Request body:
        action: action that needs to be performed
               type: enum
               valid values: [ 'down', 'reload', 'up' ]
               default: null
               required: yes
        depends: dependencies that need to be taken into account while
                 reloading the interface
                 type: enum
                 valid values: [ 'interfaces', 'vlan' ]
                 default: null
                 required: yes

        Response Body:
        Output of execute command

        Response status code:
        200: Success
        400: Bad request when param or value are invalid
        """
        action = req.params.get('action', None)
        if action == self.ACTION_RELOAD:
            options = '-a'
            cmd = '{BIN} {options}'.format(BIN=_IFRELOAD,
                                           options=options)
        elif action in [self.ACTION_UP, self.ACTION_DOWN]:
            options = ''
            if devName is None:
                raise falcon.HTTPBadRequest(falcon.HTTP_BAD_REQUEST,
                                            'Please provide a device name')
            depends = req.params.get('depends', None)
            if depends is not None:
                if depends not in self.__DEPENDS_VALUES:
                    raise falcon.HTTPBadRequest(
                        falcon.HTTP_BAD_REQUEST,
                        '"depends" can be one of %s' %
                        ', '.join(self.__DEPENDS_VALUES)
                    )
                options = '--with-depends %s' % depends
            cmd = '{BIN} {options} {dev}'.format(BIN=_IFDOWN,
                                                 options=options,
                                                 dev=devName)
        else:
            raise falcon.HTTPBadRequest(falcon.HTTP_BAD_REQUEST,
                                        '"action" can be one of %s' %
                                        ', '.join(self.__ACTION_VALUES))
        execute_command(cmd, resp)


class IpLink(object):

    PARAM_STATUS = 'status'
    PARAM_MTU = 'mtu'
    PARAM_ADDRESS = 'address'

    def on_get(self, req, resp, devName=None):
        """
        Command:
        ip -o link show

        URI (GET):
        intf/link?detail=1
        intf/link/{dev}?detail=1

        URI parameters:
        detail: display a detailed output
                type: boolean
                valid values: [ '1', 'true', 'yes', 'on' ] for True
                default: False
                required: no

        Response Body:
        Output of execute command

        Response status code:
        200: Success
        """
        options = '-o'
        detail = req.params.get('detail', '0') in ('1', 'true', 'yes', 'on')
        options += ' -d' if detail else ''
        cmd = '{BIN} {options} link show'.format(BIN=_IP,
                                                 options=options)
        if devName is not None:
            cmd += ' %s' % devName
        execute_command(cmd, resp)

    def on_post(self, req, resp, devName):
        """
        Command:
        ip link set dev {dev} {status}
        ip link set dev {dev} mtu {mtu}

        URI:
        intf/link/{dev}

        Request body:
        param: name of parameter that should be updated
               type: enum
               valid values: [ 'mtu', 'status', 'address' ]
               default: null
               required: yes
        value: parameter value
               type: string
               valid values: if param == mtu:
                                 value should be an integer
                             elif param == status:
                                 FIXME: link status
                             else::
                                 valid IP address
               default: null
               required: yes

        Response Body:
        Output of execute command

        Response status code:
        200: Success
        400: Bad request when param or value are invalid
        """
        params = req.params
        param = params.get('param', None)
        value = params.get('value', None)
        if param == self.PARAM_STATUS:
            options = ''
            target = 'link set {dev} {value}'.format(value=value,
                                                     dev=devName)
        elif param == self.PARAM_MTU:
            options = ''
            target = 'link set {dev} mtu {value}'.format(value=value,
                                                         dev=devName)
        elif param == self.PARAM_ADDRESS:
            options = ''
            target = 'addr add {value} dev {dev}'.format(value=value,
                                                         dev=devName)

        else:
            msg = 'Invald parameter'
            _LOG.error(msg)
            raise falcon.HTTPBadRequest(falcon.HTTP_400, msg)
        cmd = '{BIN} {options} {target}'.format(BIN=_IP,
                                                options=options,
                                                target=target)
        execute_command(cmd, resp)
