# Copyright 2016 Cumulus Networks, Inc. All rights reserved.

import os
import subprocess
import uuid

from oslo.config import cfg

from restapi.utils.execute import execute_command
from restapi.utils.execute import execute_command_no_resp

INVALID_PID = 0

class L2Manager(object):
    """
    Much of this code is from the Linux Bridge agent for Neutron
    """
    __BRIDGE_INTERFACES_FS = '/sys/devices/virtual/net/{bridge}/brif/'

    __BRIDGE_NAME_PREFIX = 'br'

    __VXLAN_NAME_PREFIX = 'vxlan'

    __SUBINTERFACE_NAME = '{interface}.{vlan}'

    @classmethod
    def __device_exists(cls, device_name):
        cmd = 'ip link show dev {device}'.format(device=device_name)
        result, status = execute_command_no_resp(cmd)
        if result == None:
            return False
        return status

    @classmethod
    def __interface_exists_on_bridge(cls, bridge_name, interface):
        directory = cls.__BRIDGE_INTERFACES_FS.format(bridge=bridge_name)
        if not os.path.isdir(directory):
            return False

        for filename in os.listdir(directory):
            if filename == interface:
                return True
        return False

    @classmethod
    def __up_device(cls, device_name):
        cmd = 'ip link set {device} up'.format(device=device_name)
        execute_command_no_resp(cmd)

    @classmethod
    def __down_device(cls, device_name):
        cmd = 'ip link set {device} down'.format(device=device_name)
        execute_command_no_resp(cmd)

    @classmethod
    def __delete_device(cls, device_name, resp, is_bridge):
        if not cls.__device_exists(device_name):
            return

        cls.__down_device(device_name)
        if cfg.CONF.ML2.use_net:
            if is_bridge:
                cmd = 'net del bridge {device}'.format(device=device_name)
            else:
                cmd = 'net del interface {device}'.format(device=device_name)
        else:
            cmd = 'ip link del {device}'.format(device=device_name)
        execute_command(cmd, resp)

        if cfg.CONF.ML2.use_net:
            cmd = 'net commit'
            execute_command(cmd, resp)

    @classmethod
    def get_bridge_name(cls, network_id):
        bridge_name = cls.__BRIDGE_NAME_PREFIX + network_id[:12]
        return bridge_name

    @classmethod
    def get_subinterface_name(cls, physical_interface, vlan_id):
        return cls.__SUBINTERFACE_NAME.format(interface=physical_interface,
                                              vlan=vlan_id)

    @classmethod
    def get_vxlan_name(cls, vni):
        return cls.__VXLAN_NAME_PREFIX + str(vni)

    @classmethod
    def add_vlan(cls, physical_interface, vlan_id, resp):
        interface = cls.get_subinterface_name(physical_interface, vlan_id)
        if not cls.__device_exists(interface):
            if cfg.CONF.ML2.use_net:
                cmd = ('net add interface {device} inet static'.
                       format(device=interface))
            else:
                cmd = ('ip link add link {phys} name {device} type vlan id'
                       ' {vid}'.format(phys=physical_interface,
                                       device=interface,
                                       vid=vlan_id))
            execute_command(cmd, resp)
            if cfg.CONF.ML2.use_net:
                cmd = 'net commit'
                execute_command(cmd, resp)
            cls.__up_device(interface)

        return interface

    @classmethod
    def delete_vlan(cls, interface_name, resp):
        cls.__delete_device(interface_name, resp, False)

    @classmethod
    def add_interface(cls, bridge_name, interface_name, vlan_id,
                      newbridge, pvid, resp):
        exists = False
        if cls.__interface_exists_on_bridge(bridge_name, interface_name):
            if newbridge:
                exists = True
            else:
                return
        if not exists:
            if cfg.CONF.ML2.use_net:
                cmd = ('net add bridge bridge-ports {interface}'.
                       format(bridge=bridge_name, interface=interface_name))
            else:
                cmd = ('brctl addif {bridge} {interface}'.
                       format(bridge=bridge_name, interface=interface_name))
            execute_command(cmd, resp)

        if newbridge:
            if pvid:
                cmd = ('bridge vlan del vid 1 untagged pvid dev {interface}'.
                       format(interface=interface_name))
                execute_command(cmd, resp)
                cmd = ('bridge vlan add vid {vlan} untagged pvid dev {interface}'.
                       format(vlan=vlan_id, interface=interface_name))
            else:
                cmd = ('bridge vlan add vid {vlan} dev {interface}'.
                       format(vlan=vlan_id, interface=interface_name))
            execute_command(cmd, resp)

        if cfg.CONF.ML2.use_net:
            cmd = 'net commit'
            execute_command(cmd, resp)

    @classmethod
    def delete_interface(cls, bridge_name, physical_interface, vlan_id, newbridge, is_delete, resp):
        if newbridge:
            interface_name = physical_interface
        else:
            interface_name = cls.get_subinterface_name(physical_interface, vlan_id)

        if not cls.__interface_exists_on_bridge(bridge_name, interface_name):
            return interface_name

        if newbridge:
            cmd = ('bridge vlan del vid {vlan} dev {interface}'.
                   format(vlan=vlan_id, interface=interface_name))
            execute_command(cmd, resp)

        if (not newbridge) or is_delete:
            if cfg.CONF.ML2.use_net:
                cmd = ('net del bridge {bridge} bridge-ports {interface}'.
                       format(bridge=bridge_name, interface=interface_name))
            else:
                cmd = ('brctl delif {bridge} {interface}'.
                       format(bridge=bridge_name, interface=interface_name))
            execute_command(cmd, resp)

        if cfg.CONF.ML2.use_net:
            cmd = 'net commit'
            execute_command(cmd, resp)
        return interface_name

    @classmethod
    def add_bridge(cls, bridge_name, vlan_id, spf, newbridge, resp):
        bridge_exists = False
        if cls.__device_exists(bridge_name):
            if not newbridge:
                return
            else:
                bridge_exists = True

        if newbridge:
            if not cfg.CONF.ML2.use_net:
                if not bridge_exists:
                    cmd = 'brctl addbr {bridge}'.format(bridge=bridge_name)
                    execute_command(cmd, resp)

                    cmd = 'ip link set {bridge} type bridge vlan_filtering 1'\
                            .format(bridge=bridge_name)
                    execute_command(cmd, resp)

                    if spf:
                        cmd = 'brctl stp {bridge} on'.format(bridge=bridge_name)
                    else:
                        cmd = 'brctl stp {bridge} off'.format(bridge=bridge_name)
                    execute_command(cmd, resp)

                cmd = 'bridge vlan add vid {vlan} dev {bridge} self'\
                        .format(vlan=vlan_id, bridge=bridge_name)
                execute_command(cmd, resp)

            else:
                if not bridge_exists:
                    if spf:
                        cmd = 'net add bridge bridge-stp yes'
                    else:
                        cmd = 'net add bridge bridge-stp no'

                    execute_command(cmd, resp)

                cmd = 'net add bridge bridge-vids {vlan}'.format(vlan=vlan_id)
                execute_command(cmd, resp)

                cmd = 'net commit'
                execute_command(cmd, resp)

        else:
            cmd = 'brctl addbr {bridge}'.format(bridge=bridge_name)
            execute_command(cmd, resp)
            if spf:
                cmd = 'brctl stp {bridge} on'.format(bridge=bridge_name)
            else:
                cmd = 'brctl stp {bridge} off'.format(bridge=bridge_name)
            execute_command(cmd, resp)

        if cfg.CONF.ML2.trunk_interfaces:
            for trunk in cfg.CONF.ML2.trunk_interfaces:
                if newbridge:
                    trunk_interface_name = trunk
                else:
                    trunk_interface_name = cls.add_vlan(trunk, vlan_id, resp)
                cls.add_interface(bridge_name, trunk_interface_name,
                                  vlan_id, newbridge, False, resp)
        cls.__up_device(bridge_name)

    @classmethod
    def delete_bridge(cls, bridge_name, vlan_id, newbridge, refcnt, resp):
        if newbridge:
            cmd = 'bridge vlan del vid {vlan} dev {bridge} self'\
                        .format(vlan=vlan_id, bridge=bridge_name)
            execute_command(cmd, resp)

        if refcnt == 1:
            cls.__delete_device(bridge_name, resp, True)
            if (not newbridge) and cfg.CONF.ML2.trunk_interfaces:
                for trunk in cfg.CONF.ML2.trunk_interfaces:
                    interface_name = cls.get_subinterface_name(trunk, vlan_id)
                    cls.delete_vlan(interface_name, resp)
        else:
            if newbridge:
                for trunk in cfg.CONF.ML2.trunk_interfaces:
                    cls.delete_interface(bridge_name, trunk, vlan_id, newbridge, False, resp)

    @classmethod
    def add_vxlan(cls, vni, resp):
        vxlan_name = cls.get_vxlan_name(vni)

        if cls.__device_exists(vxlan_name):
            return vxlan_name

        cmd = ('ip link add {vxlan} type vxlan id {vni} local {bind_ip} '
               'remote {node_ip}'.format(vxlan=vxlan_name,
                                         vni=vni,
                                         bind_ip=cfg.CONF.ML2.local_bind,
                                         node_ip=cfg.CONF.ML2.service_node))
        execute_command(cmd, resp)

        cls.__up_device(vxlan_name)

        return vxlan_name

    @classmethod
    def delete_vxlan(cls, vni, resp):
        vxlan_name = cls.get_vxlan_name(vni)
        if not cls.__device_exists(vxlan_name):
            return

        cls.__down_device(vxlan_name)
        cmd = 'ip link del {device}'.format(device=vxlan_name)
        execute_command(cmd, resp)


class HashManager(object):

    def __init__(self):
        self.pid = os.getpid()
        self.net_pid = self.get_networking_pid()
        self.hashid = uuid.uuid1()

    def get_networking_pid(self):
        output = subprocess.check_output("systemctl status networking.service",
                                         shell=True)
        for row in output.split('\n'):
            if 'Main PID' in row:
                key, value = row.split(': ')
                pid, info = value.split(' (')
                return int(pid)

        return INVALID_PID

    def get_hash_id(self):
        curr_net_pid = self.get_networking_pid()
        if curr_net_pid != self.net_pid:
            self.hashid = uuid.uuid1()
        return str(self.hashid)
