# vim: tabstop=4 shiftwidth=4 softtabstop=4
# Copyright 2016 Cumulus Networks, Inc. All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc.
# 51 Franklin Street, Fifth Floor
# Boston, MA  02110-1301, USA.
from __future__ import absolute_import

import json

import falcon
try:
    from netq.lib.netq import (
        NetQ,
        NetQException
    )
except ImportError:
    pass

from restapi.utils.logs import get_logger

_LOG = get_logger()


def _get_netq_obj(req):
    server_addr = req.get_param('serverAddress')
    timestamp = req.get_param_as_int('timestamp')
    return NetQ(server_addr, timestamp)


def _message(msg, jsonify=True):
    out = {'message': msg}
    return json.dumps(out) if jsonify else out


def _tabulate(output, keys, jsonify=True):
    out = []
    for ele in output:
        out.append(dict((key, item) for key, item in zip(keys, ele)))
    return json.dumps(out) if jsonify else out


class NetQShowAddresses(object):

    def on_get(self, req, resp):
        node = req.get_param('hostname')
        iface = req.get_param('ifName')
        ip_address = req.get_param('ipAddress')
        netq = _get_netq_obj(req)
        out = netq.show_addresses(node, iface, ip_address)
        if out:
            resp.status = falcon.HTTP_200
            resp.body = _tabulate(out, ['ipAddress', 'hostname', 'ifName',
                                        'updateTime'])
        else:
            resp.status = falcon.HTTP_404
            resp.body = _message('No matching Interface Address entry found')


class NetQShowAsn(object):

    def on_get(self, req, resp):
        asn = req.get_param('asn')
        netq = _get_netq_obj(req)
        out = netq.show_asn(asn)
        if out:
            resp.status = falcon.HTTP_200
            resp.body = _tabulate(out, ['asn', 'hostname', 'routerId'])
        else:
            resp.status = falcon.HTTP_404
            resp.body = _message('No matching ASN record found')


class NetQShowBgpStatus(object):

    def on_get(self, req, resp):
        netq = _get_netq_obj(req)
        out = netq.show_bgp_status()
        resp.status = falcon.HTTP_200
        if out:
            resp.body = _tabulate(out, ['hostname', 'peer', 'reason'])
        else:
            resp.body = json.dumps([])


class NetQShowClagStatus(object):

    def on_get(self, req, resp):
        netq = _get_netq_obj(req)
        out = netq.show_clag_status()
        resp.status = falcon.HTTP_200
        if out:
            resp.body = _tabulate(out, ['hostname'])
        else:
            resp.body = json.dumps([])


class NetQShowIPNeighbors(object):

    def on_get(self, req, resp):
        ip_address = req.get_param('ipAddress')
        netq = _get_netq_obj(req)
        out = netq.show_ip_neighbors(ip_address)
        if out:
            resp.status = falcon.HTTP_200
            resp.body = _tabulate(out, ['ipAddress', 'hostname', 'ifName',
                                        'macAddress', 'updateTime'])
        else:
            resp.status = falcon.HTTP_404
            resp.body = _message('No IP neighbor entry found')


class NetQShowLinks(object):

    def on_get(self, req, resp):
        netq = _get_netq_obj(req)
        out = netq.show_links()
        if out:
            resp.status = falcon.HTTP_200
            resp.body = _tabulate(out, ['hostname', 'ifName', 'state', 'type',
                                        'lldp', 'updateTime'])
        else:
            resp.body = json.dumps([])


class NetQShowMacs(object):

    def on_get(self, req, resp):
        mac_address = req.get_param('macAddress')
        vlan = req.get_param_as_int('vlan')
        netq = _get_netq_obj(req)
        out = netq.show_macvlan(mac_address, vlan)
        if out:
            resp.status = falcon.HTTP_200
            resp.body = _tabulate(out, ['macAddress', 'vlan', 'hostname',
                                        'egressPort', 'updateTime'])
        else:
            resp.status = falcon.HTTP_404
            resp.body = _message('No matching MAC entry found')


class NetQShowNeighbors(object):

    def on_get(self, req, resp):
        hostname = req.get_param('hostname')
        iface = req.get_param('ifName')
        netq = _get_netq_obj(req)
        out = netq.show_neighbors(hostname, iface)
        if out:
            resp.status = falcon.HTTP_200
            resp.body = _tabulate(out, ['hostname', 'ifName', 'lldpPeer',
                                        'peerIntf', 'updateTime'])
        else:
            resp.status = falcon.HTTP_404
            resp.body = _message('No LLDP record found')


class NetQShowRouterId(object):

    def on_get(self, req, resp):
        router_id = req.get_param('routerId')
        netq = _get_netq_obj(req)
        out = netq.show_router_id(router_id)
        if out:
            resp.status = falcon.HTTP_200
            resp.body = _tabulate(out,
                                  ['routerId', 'hostname', 'asn', 'protocol'])
        else:
            resp.status = falcon.HTTP_404
            resp.body = _message('No matching routerId record found')


class NetQShowRoutes(object):

    def on_get(self, req, resp):
        netq = _get_netq_obj(req)
        prefix = req.get_param('prefix')
        out = netq.show_routes(prefix)
        if out:
            resp.status = falcon.HTTP_200
            resp.body = _tabulate(sorted(out),
                                  ['ipAddress', 'hostname', 'nextHops',
                                   'origin', 'updateTime'])
        else:
            resp.status = falcon.HTTP_404
            resp.body = _message('No matching routes found')


class NetQShowStatus(object):

    def on_get(self, req, resp):
        netq = _get_netq_obj(req)
        out = netq.show_status()
        resp.status = falcon.HTTP_200
        resp.body = _tabulate(out,
                              ['hostname', 'connectTime', 'lastConnect'])


class NetQTopTalkersRx(object):

    def on_get(self, req, resp):
        netq = _get_netq_obj(req)
        count = req.get_param_as_int('count') or 5
        out, drop_nodes = netq.top_talkers_rx(count)
        resp.status = falcon.HTTP_200
        resp.body = json.dumps([{
            'counters': _tabulate(sorted(out),
                                  ['hostname', 'ifName', 'bytes'],
                                  jsonify=False),
            'dropNodes': _tabulate(sorted(drop_nodes),
                                   ['hostname', 'ifName', 'bytes'],
                                   jsonify=False)
        }])


class NetQTopTalkersTx(object):

    def on_get(self, req, resp):
        netq = _get_netq_obj(req)
        count = req.get_param_as_int('count') or 5
        out, drop_nodes = netq.top_talkers_tx(count)
        resp.status = falcon.HTTP_200
        resp.body = json.dumps([{
            'counters': _tabulate(sorted(out),
                                  ['hostname', 'ifName', 'bytes'],
                                  jsonify=False),
            'dropNodes': _tabulate(sorted(drop_nodes),
                                   ['hostname', 'ifName', 'bytes'],
                                   jsonify=False)
        }])


class NetQView(object):

    def on_get(self, req, resp, hostname):
        key = req.get_param('key', required=True)
        netq = _get_netq_obj(req)
        try:
            resp.status = falcon.HTTP_200
            resp.body = json.dumps([
                netq.view(hostname, *key.split(' '))
            ])
        except NetQException as ex:
            resp.status = falcon.HTTP_404
            resp.body = _message(str(ex))