# vim: tabstop=4 shiftwidth=4 softtabstop=4
# Copyright 2016 Cumulus Networks, Inc. All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc.
# 51 Franklin Street, Fifth Floor
# Boston, MA  02110-1301, USA.
from __future__ import absolute_import

import inspect
import json
import sys

import falcon
from oslo.config import cfg

_REDIS_MODELS = []
try:
    import netq.orm.redisdb.models
    from netq.orm.redisdb import set_connection_settings
except ImportError:
    # This is to avoid mandating the netq package
    pass
else:
    for _, obj in inspect.getmembers(sys.modules['netq.orm.redisdb.models']):
        if inspect.isclass(obj):
            _REDIS_MODELS.append(obj)

from restapi.utils.logs import get_logger

_LOG = get_logger()


def _validate_model(model_name):
    model_class = next((model
                        for model in _REDIS_MODELS
                        if model_name == model.__name__), None)
    if model_class is None:
        raise falcon.HTTP_404('Model %s not found' % model_name)
    set_connection_settings(cfg.CONF.NETQ.server_address)
    return model_class


class _Params(object):

    __FIRST = 'first'
    __END = 'end'
    __KEY_TYPE = 'key_type'
    __REVERSE = 'reverse'
    __START = 'start'
    __TIMESTAMP = 'timestamp'
    ALL = [__FIRST, __KEY_TYPE, __TIMESTAMP, __REVERSE, __START, __END]

    def __init__(self, req):
        self.params = req.params
        self.key_type = req.get_param_as_int(self.__KEY_TYPE)
        self.timestamp = req.get_param_as_int(self.__TIMESTAMP)
        self.start = req.get_param_as_int(self.__END)
        self.end = req.get_param_as_int(self.__START)
        self.reverse = req.get_param_as_bool(self.__REVERSE)
        self.first = req.get_param_as_bool(self.__FIRST)
        self.__delete_params()

    def __delete_params(self):
        for param in self.ALL:
            self.params.pop(param, None)


class Models(object):

    def on_get(self, _, resp):
        resp.status = falcon.HTTP_200
        resp.body = json.dumps([model.__name__ for model in _REDIS_MODELS])


class QueryAll(object):

    def on_get(self, req, resp, modelName):
        model_class = _validate_model(modelName)
        params = _Params(req)
        resp.status = falcon.HTTP_200
        try:
            json_output = [
                obj.to_json() for obj in
                model_class.query.all(key_type=params.key_type,
                                      timestamp=params.timestamp)
            ]
            resp.body = json.dumps(json_output)
        except Exception as ex:
            resp.status = falcon.HTTP_500
            _LOG.error('Query failed. out:%s', ex)


class QueryCount(object):

    def on_get(self, req, resp, modelName):
        model_class = _validate_model(modelName)
        keys = req.get_param('keys').split(',')
        resp.status = falcon.HTTP_200
        try:
            json_output = [obj for obj in
                           model_class.query.count(keys=keys)]
            print json_output
            resp.body = json.dumps(json_output)
        except Exception as ex:
            resp.status = falcon.HTTP_500
            _LOG.error('Query failed. out:%s', ex)


class QueryDelete(object):

    def on_delete(self, req, resp, modelName):
        model_class = _validate_model(modelName)
        params = _Params(req)
        purge = req.get_param_as_bool('purge')
        resp.status = falcon.HTTP_200
        try:
            model_class.query.delete(key_type=params.key_type,
                                     purge=purge,
                                     **params.params)
        except Exception as ex:
            resp.status = falcon.HTTP_500
            _LOG.error('Query failed. out:%s', ex)


class QueryFilter(object):

    def on_get(self, req, resp, modelName):
        model_class = _validate_model(modelName)
        params = _Params(req)
        resp.status = falcon.HTTP_200
        try:
            json_output = [
                obj.to_json() for obj in
                model_class.query.filter(key_type=params.key_type,
                                         timestamp=params.timestamp,
                                         **params.params)
            ]
            resp.body = json.dumps(json_output)
        except Exception as ex:
            resp.status = falcon.HTTP_500
            _LOG.error('Query failed. out:%s', ex)


class QueryGet(object):

    def on_get(self, req, resp, modelName):
        model_class = _validate_model(modelName)
        params = _Params(req)
        resp.status = falcon.HTTP_200
        try:
            obj = model_class.query.get(key_type=params.key_type,
                                        timestamp=params.timestamp,
                                        **params.params)
            if obj is not None:
                resp.body = json.dumps(obj.to_json())
        except Exception as ex:
            resp.status = falcon.HTTP_500
            _LOG.error('Query failed. out:%s', ex)


class QueryLpm(object):

    def on_get(self, req, resp, modelName):
        model_class = _validate_model(modelName)
        params = _Params(req)
        prefix = req.get_param('prefix', required=True)
        hostnames = req.get_param('hostnames', required=True).split(',')
        is_ipv6 = req.get_param_as_bool('is_ipv6', required=True)
        _LOG.info('Hostnames are %s', hostnames)
        resp.status = falcon.HTTP_200
        try:
            json_output = [
                obj.to_json()
                for obj in model_class.query.lpm(
                    prefix, hostnames, timestamp=params.timestamp,
                    is_ipv6=is_ipv6
                )
            ]
            resp.body = json.dumps(json_output)
        except Exception as ex:
            resp.status = falcon.HTTP_500
            _LOG.error('Query failed. out:%s', ex)


class QueryRange(object):

    def on_get(self, req, resp, modelName):
        model_class = _validate_model(modelName)
        params = _Params(req)
        resp.status = falcon.HTTP_200
        try:
            json_output = [
                obj.to_json() for obj in
                model_class.query.range(key_type=params.key_type,
                                        start=params.start or -1,
                                        end=params.end or -1,
                                        reverse=params.reverse or False,
                                        first=params.first or True,
                                        **params.params)
            ]
            resp.body = json.dumps(json_output)
        except Exception as ex:
            resp.status = falcon.HTTP_500
            _LOG.error('Query failed. out:%s', ex)


class QueryRangeByScore(object):

    def on_get(self, req, resp, modelName):
        model_class = _validate_model(modelName)
        params = _Params(req)
        resp.status = falcon.HTTP_200
        try:
            json_output = [
                obj.to_json() for obj in
                model_class.query.range(key_type=params.key_type,
                                        start=params.start or -1,
                                        end=params.end or -1,
                                        reverse=params.reverse or False,
                                        first=params.first or True,
                                        **params.params)
            ]
            resp.body = json.dumps(json_output)
        except Exception as ex:
            resp.status = falcon.HTTP_500
            _LOG.error('Query failed. out:%s', ex)