import os
import sys

from gunicorn import util
from gunicorn.app.base import BaseApplication
from oslo.config import cfg
import six

from restapi import settings
from restapi.utils.config import setup_conf
from restapi.utils.logs import get_logger, setup_logging

_LOG = get_logger()


class _RestApiApplication(BaseApplication):
    """ Custom Gunicorn Application
    """
    def __init__(self):
        """ Load the base config and assign some core attributes.
        """
        setup_conf()
        setup_logging()
        super(_RestApiApplication, self).__init__()

    @staticmethod
    def get_config_from_filename(filename):
        gunicorn_cfg = {
            '__builtins__': __builtins__,
            '__file__': filename,
            '__name__': '__config__',
            '__doc__': None,
            '__package__': None
        }
        try:
            with open(filename) as f:
                try:
                    code = compile(f.read(), filename, 'exec')
                    six.exec_(code, gunicorn_cfg, gunicorn_cfg)
                except:
                    _LOG.error('Failed to read config file: %s', filename)
                    raise
        except OSError:
            _LOG.error('Failed to open config file: %s', filename)
        return gunicorn_cfg

    def load_config_from_filename(self, filename):
        gunicorn_cfg = self.get_config_from_filename(filename)
        for k, v in gunicorn_cfg.items():
            # Ignore unknown names
            if k not in self.cfg.settings:
                continue
            try:
                self.cfg.set(k.lower(), v)
            except:
                _LOG.error('Invalid value for %s: %s' % (k, v))
                raise

    def load_config(self):
        self.init()
        if os.path.exists(cfg.CONF.server_config):
            self.load_config_from_filename(cfg.CONF.server_config)

    def load(self):
        return util.import_app('wsgi:api')

    def init(self, *args):
        self.cfg.set('bind', cfg.CONF.bind)
        self.cfg.set('pidfile', (
            self.cfg.pidfile or cfg.CONF.pidfile
        ))
        self.cfg.set('daemon', cfg.CONF.daemon)
        if settings.DEVELOPMENT:
            self.cfg.set('reload', True)
            self.cfg.set('timeout', 120)

    def run(self):
        try:
            self.load()
        except:
            _LOG.error('Error while loading the application')
            raise

        if self.cfg.daemon:
            util.daemonize(self.cfg.enable_stdio_inheritance)

        # set python paths
        if self.cfg.pythonpath and self.cfg.pythonpath is not None:
            paths = self.cfg.pythonpath.split(',')
            for path in paths:
                pythonpath = os.path.abspath(path)
                if pythonpath not in sys.path:
                    sys.path.insert(0, pythonpath)
        super(_RestApiApplication, self).run()


def main():
    app = _RestApiApplication()
    return app.run()