import json
import os

from restapi import settings
from restapi.utils.logs import get_logger
from restapi.utils.misc import get_callable

_LOG = get_logger()


class ServeSpec(object):

    def __init__(self, spec_path):
        self.__spec_path = spec_path

    def on_get(self, req, resp):
        resp.content_type = 'application/json'
        resp.stream = open(self.__spec_path, 'rb')
        return resp


class Swagger(object):

    def __init__(self, app, app_name, spec_file):
        self.__app = app
        self.__app_name = app_name
        self.__spec_file = spec_file
        self.__spec = None

    def __get_method_endpoint(self, method_attrs):
        return '%s.%s.%s.%s' % (settings.APPS,
                                self.__app_name,
                                method_attrs['tags'][0],
                                method_attrs['operationId'])

    def __load_spec(self):
        _LOG.info('Loading swagger spec file')
        if not os.path.exists(self.__spec_file):
            msg = 'Swagger spec file %s not found' % self.__spec_file
            _LOG.error(msg)
            raise RuntimeError(msg)
        try:
            with open(self.__spec_file, 'r') as spec_fd:
                try:
                    spec_dict = json.load(spec_fd)
                except Exception:
                    _LOG.exception('Failed to parse the swagger spec %s',
                                   self.__spec_file)
                    raise
                return spec_dict
        except Exception:  # pylint: disable=broad-except
            raise RuntimeError('Failed to load swagger spec')

    @property
    def spec(self):
        if self.__spec is None:
            self.__spec = self.__load_spec()
        return self.__spec

    def add_routes(self):
        base_path = self.spec['basePath']
        _LOG.info('Serving %s from %s', base_path, self.__spec_file)
        self.__app.add_route(base_path, ServeSpec(self.__spec_file))
        for path, method_map in self.spec['paths'].items():
            for method, attrs in method_map.items():
                operation = get_callable(self.__get_method_endpoint(attrs))
                try:
                    url_path = '%s%s' % (base_path, path)
                    _LOG.debug('adding route %s', url_path)
                    self.__app.add_route(url_path, operation())
                except Exception:  # pylint: disable=broad-except
                    msg = ('Unable to build routing table from provided '
                           'Swagger spec.')
                    _LOG.error(msg)
